/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gPostObserver;

function composerOnload() {
  
    if (!("@mozilla.org/editor/htmleditor;1" in Components.classes)) {
        window.alert ("HTML editor not installed. (This functionality is not yet supported in Phoenix.)");
        window.close();
    }
    if (window.arguments && (window.arguments.length == 4)) {
      try {
          gPostObserver = new CommonEditor(
              window.arguments[0],
              window.arguments[1],
              window.arguments[2],
              window.arguments[3]
          );
      } catch (e) {
          window.alert ("Failed to set up composer window. Error: " + e);
      }
    }
    else {
	window.alert("Improper arguments passed to composer.xul");
	window.close();
    }
}

// Get an editorWrapper object appropriate to the Mozilla version
function getEditorWrapper () {
    if (document.getElementById("edit-frame").editorShell) {
        return new EditorWrapperV1();
    } else {
        return new EditorWrapperV2();
    }
}


// For Mozilla versions before 1.3
function EditorWrapperV1 () {
    this.editorShell = document.getElementById("edit-frame").editorShell;
    this.editorShell.editorType = 'html';
    this.editorShell.webShellWindow = window.frames[0];
    this.editorShell.contentWindow  = window.frames[0];
    this.editorShell.contentsMIMEType = "text/html";
}

EditorWrapperV1.prototype.loadUrl = function (url) {
    this.editorShell.LoadUrl(url);
}

EditorWrapperV1.prototype.tidy = function (doc) {
  var s = Components.classes[PostObserver.XMLSERIALIZER_CONTRACTID].createInstance(Components.interfaces.nsIDOMSerializer);
  var xml = s.serializeToString(doc);
  
  // remove doctype
  xml = xml.replace (/<!DOCTYPE[^>]*>/,'');
  
  // remove _moz attributes
  xml = xml.replace (/ _moz_[a-z_]*="[^"]*"/g,'');

  // remove type="_moz" attribute
  xml = xml.replace (/type="_moz"/g,'');

  // lowercase the tag names
  xml = xml.replace (/<([a-zA-Z0-9\/]+)([^]*?)>/g, function(wholeMatch, tagName, restOfTag) { return "<" + tagName.toLowerCase() + restOfTag + ">"; });

  // add a namespace
  xml = xml.replace (/<html[^]*?>/, '<html xmlns="http://www.w3.org/1999/xhtml">');

  return xml;
}

EditorWrapperV1.prototype.getDocument = function () {
    // Need to coerce the edited document into well-formed XML
    // (so editorShell.GetContentsAs('text/html', 0) won't do)
    return this.tidy (this.editorShell.editorDocument);
}

// For Mozilla versions 1.3
// Code adapted from Mozblog
function EditorWrapperV2 () {
    this.editorElement = document.getElementById("edit-frame");
}

EditorWrapperV2.prototype.loadUrl = function (url) {
    this.editorElement.webNavigation.loadURI
        (url,
         nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE, // load flags
         null,                                     // referrer
         null,                                     // post-data stream
         null);
}

EditorWrapperV2.prototype.tidy = function (doc) {
  var s = Components.classes[PostObserver.XMLSERIALIZER_CONTRACTID].createInstance(Components.interfaces.nsIDOMSerializer);
  var xml = s.serializeToString(doc);
  
  // remove doctype
  xml = xml.replace (/<!DOCTYPE[^>]*>/,'');
  
  // remove _moz attributes
  xml = xml.replace (/ _moz_[a-z_]*="[^"]*"/g,'');

  // remove type="_moz" attribute
  xml = xml.replace (/type="_moz"/g,'');

  // lowercase the tag names
  xml = xml.replace (/<([a-zA-Z0-9\/]+)([^]*?)>/g, function(wholeMatch, tagName, restOfTag) { return "<" + tagName.toLowerCase() + restOfTag + ">"; });

  // add a namespace
  xml = xml.replace (/<html[^]*?>/, '<html xmlns="http://www.w3.org/1999/xhtml">');

  return xml;
}


EditorWrapperV2.prototype.getDocument = function () {
    // Need to coerce the edited document into well-formed XML
    // (so editorShell.GetContentsAs('text/html', 0) won't do)
    return this.tidy (this.editorElement.getEditor(this.editorElement.contentWindow).document);
}
